from crypt import crypt
from string import ascii_letters, digits
from random import choice
from collections import OrderedDict

from exceptions import HtException


class UserExists(HtException):

    def __str__(self):
        return "User already exists"


class UserNotExists(HtException):

    def __str__(self):
        return "User not exists"


class Basic(object):
    """ Basic object deals with Basic HTTP Authorization configuration file.
    It is passed the path to userdb file. """

    def __init__(self, userdb):
        self.userdb = userdb
        self.users = OrderedDict()
        self.new_users = OrderedDict()

    def __enter__(self):
        with open(self.userdb, "r") as users:
            for i in users:
                user, password = i.split(":", 1)
                self.users[user] = password
        self.new_users = self.users.copy()
        return self

    def __exit__(self, type, value, traceback):
        if self.new_users == self.users:
            return
        with open(self.userdb, "w") as userdb:
            for user in self.new_users:
                userdb.write("%s:%s" % (user, self.new_users[user]))

    def show_users(self):
        """ Returns users in a tuple """
        return self.new_users.keys()

    def is_user_exists(self, user):
        """ Returns True if user exists """
        return user in self.show_users()

    def add_user(self, user, password):
        """ Adds a user with password """
        if self.is_user_exists(user):
            raise UserExists
        self.new_users[user] = self._crypt_password(password) + "\n"

    def delete_user(self, user):
        """ Deletes a user """
        if not self.is_user_exists(user):
            raise UserNotExists
        self.new_users.pop(user)

    def change_password(self, user, password):
        """ Changes user password """
        if not self.is_user_exists(user):
            raise UserNotExists
        self.new_users[user] = self._crypt_password(password) + "\n"

    def _crypt_password(self, password):
        """ Crypts password """

        def salt():
            """ Generates some salt """
            symbols = ascii_letters + digits
            return choice(symbols) + choice(symbols)

        return crypt(password, salt())



import re
from orderedmultidict import omdict

from exceptions import HtException


class GroupNotExists(HtException):
    def __str__(self):
        return "Group not exists"


class UserAlreadyInAGroup(HtException):
    def __str__(self):
        return "User already in a group"


class UserNotInAGroup(HtException):
    def __str__(self):
        return "User not in a group"


class Group(object):
    """ Group object deals with group authorization files.  It is passed the
    path to groupdb file. """
    def __init__(self, groupdb):
        self.groupdb = groupdb
        self.groups = omdict()
        self.new_groups = omdict()

    def __enter__(self):
        with open(self.groupdb, "r") as groupdb:
            groupdb = re.sub("\\\\\n", "", groupdb.read())
            for group in groupdb.splitlines():
                groupname, users = group.split(": ", 1)
                for user in users.split():
                    self.groups.add(groupname, user)
            self.new_groups = self.groups.copy()
            return self

    def __exit__(self, type, value, traceback):
        if self.new_groups == self.groups:
            return
        with open(self.groupdb, "w") as userdb:
            for group in self.new_groups:
                userdb.write("%s: %s\n" %
                              (group, " ".join(self.new_groups.getlist(group))))

    def show_groups(self):
        """ Returns groups in a tuple """
        return self.new_groups.keys()

    def is_group_exists(self, group):
        """ Returns True if group exists """
        return group in self.show_groups()

    def is_user_in_a_group(self, user, group):
        """ Returns True if user is in a group """
        return user in self.new_groups.getlist(group)

    def add_user_to_group(self, user, group):
        """ Adds user to a group """
        if self.is_user_in_a_group(user, group):
            raise UserAlreadyInAGroup
        self.new_groups.add(group, user)

    def delete_user_from_group(self, user, group):
        """ Deletes user from group """
        if not self.is_group_exists(group):
            raise GroupNotExists
        if not self.is_user_in_a_group(user, group):
            raise UserNotInAGroup
        self.new_groups.popvalue(group, user)



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name="htpasswd",
    version="1.0",
    packages=["htpasswd"],
    install_requires=["orderedmultidict>=0.7"],
    author="Ilya A. Otyutskiy",
    author_email="sharp@thesharp.ru",
    maintainer="Ilya A. Otyutskiy",
    url="https://github.com/thesharp/htpasswd",
    description="Library to work with htpasswd user (basic authorization) "
                "and group files.",
    license="MIT"
)
